(function (Drupal, $, once) {
  Drupal.behaviors.fingerprint_timeline = {
    attach: function (context, settings) {
      $(once('fingerprint_timeline', '#timeline', context)).each(function () {

        const csrfTokenLoader = async () => {
          await axios.get(Drupal.url('session/token')).then((response) => {
            csrfToken = response.data;
          }).then(() => {

          });
        }

        const actionAddFingerprint = async (data) => {
          try {
            const response = await axios.post(Drupal.url('api/fingerprint'), data, {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': csrfToken
              }
            });
            return response.data;
          } catch (error) {
            return false
          }
        }

        const actionEditFingerprint = async (id, data) => {
          id = id.replace('fingerprint-', '')
          try {
            const response = await axios.patch(Drupal.url('api/fingerprint/' + id), data, {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': csrfToken
              }
            });
            return response.data;
          } catch (error) {
            return false
          }
        }

        const actionDeleteFingerprint = async (id) => {
          id = id.replace('fingerprint-', '')
          try {
            const response = await axios.delete(Drupal.url('api/fingerprint/' + id), {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': csrfToken
              }
            });
            return true;
          } catch (error) {
            return false
          }
        }

        const calculatorTimeline = (data, groups) => {
          data.sort((a, b) => a.start - b.start);
          let results = {};
          let dataTable = [];
          let dataWokTime = [];
          if (groups.length) {
            for (let i = 0; i < groups.length; i++) {
              let data_process = data.filter(item => item.group == groups[i].id && item.type !== 'background')
              let data_group = data.filter(item => item.group == groups[i].id && item.type == 'background')
              if (data_process.length) {
                results[groups[i].id] = processTimelineShift(data_process, data_group)
              }
            }
          }
          return results
        }

        const processTimeline = (data) => {
          let results = {};
          let session = null;
          for (let i = 0; i < data.length; i++) {
            let timePoint = new Date(data[i].start);
            let timePointNext = data?.[i + 1]?.start ? new Date(data[i + 1].start) : null;

            let date_str = moment(new Date(timePoint)).format('YYYY-MM-DD');
            if (session) {
              date_str = moment(new Date(session['start'])).format('YYYY-MM-DD');
            }

            if (!session) {
              const hours8 = 28800000; // 8 hours;
              let timePointEnd = new Date(data[i].start + hours8);
              const hours16 = 57600000; // 16 hours;
              let timePointEndMax = new Date(data[i].start + hours16);

              const minutes30 = 1800000; // 30 minutes;
              let margin_after_start = new Date(data[i].start + minutes30);

              let margin_before_end = new Date(timePointEnd.valueOf() - minutes30);
              let margin_after_end = new Date(timePointEnd.valueOf() + minutes30);

              let margin_before_end_max = new Date(timePointEndMax.valueOf() - minutes30);
              let margin_after_end_max = new Date(timePointEndMax.valueOf() + minutes30);

              session = {
                'start': timePoint,
                'end': timePointEnd,
                'end_max': timePointEndMax,
                'margin_after_start': margin_after_start,
                'margin_before_end': margin_before_end,
                'margin_after_end': margin_after_end,
                'margin_before_end_max': margin_before_end_max,
                'margin_after_end_max': margin_after_end_max,
              };
            }

            if (session) {
              if (!results?.[date_str]?.['checkin']) {
                if (!results?.[date_str]) {
                  results[date_str] = {};
                }
                results[date_str]['checkin'] = [];
              }
              if (timePoint.valueOf() <= session['margin_after_start'].valueOf()) {
                results[date_str]['checkin'].push(data[i]);
              }

              if (!results?.[date_str]?.['break']) {
                if (!results?.[date_str]) {
                  results[date_str] = {};
                }
                results[date_str]['break'] = [];
              }
              if (session['margin_after_start'].valueOf() < timePoint.valueOf() && timePoint.valueOf() < session['margin_before_end'].valueOf()) {
                results[date_str]['break'].push(data[i]);
              }

              if (!results?.[date_str]?.['checkout']) {
                if (!results?.[date_str]) {
                  results[date_str] = {};
                }
                results[date_str]['checkout'] = [];
              }
              if (session['margin_before_end'].valueOf() <= timePoint.valueOf() && timePoint.valueOf() <= session['margin_after_end_max'].valueOf()) {
                results[date_str]['checkout'].push(data[i]);
              }

              if (!timePointNext || timePointNext.valueOf() > session['margin_after_end_max'].valueOf()) {
                session = null;
                if (!results[date_str]['checkout'].length && results[date_str]['break'].length) {
                  results[date_str]['checkout'].push(results[date_str]['break'].pop());
                }
              }
            }
          }
          return results;
        }

        const processTimelineShift = (data, group) => {
          let results = {};
          let session = null;
          let marginMinutesDefault = 1800000

          for (let i = 0; i < data.length; i++) {
            let timePoint = new Date(data[i].start);
            let timePointNext = data?.[i + 1]?.start ? new Date(data[i + 1].start) : null;

            if (!session) {
              let dayStart = new Date(data[i].start);
              dayStart.setHours(0, 0, 0, 0);

              let shifts = group.filter(
                item => item.start >= dayStart.valueOf() && item.start < dayStart.valueOf() + 86400000
              );

              let shift = closestShift(shifts, data[i].start);
              const marginMinutesStart = shift ? shift['margin_start'] : marginMinutesDefault;

              let maxShift = shift ? new Date(shift.start + (86400000 - marginMinutesStart)) : null;
              let timePointEnd = shift ? new Date(shift['end']) : null;
              let timePointEndMax = maxShift;

              let marginAfterStart = shift
                ? new Date(shift.start + marginMinutesStart)
                : new Date(timePoint.valueOf() + marginMinutesStart);

              let marginBeforeEnd = timePointEnd ? new Date(timePointEnd.valueOf() - marginMinutesStart) : null;
              let marginAfterEnd = timePointEnd ? new Date(timePointEnd.valueOf() + marginMinutesStart) : null;

              session = {
                id: crypto.randomUUID(),
                start: timePoint,
                end: timePointEnd,
                end_max: timePointEndMax,
                margin_after_start: marginAfterStart,
                margin_before_end: marginBeforeEnd,
                margin_after_end: marginAfterEnd,
                point_max_shift: timePointEndMax,
              };
            }

            if (session) {

              if (session['end']) {
                results[session.id] = results[session.id] || { checkin: [], break: [], checkout: [] };

                if (timePoint.valueOf() <= session['margin_after_start'].valueOf()) {
                  results[session.id]['checkin'].push(data[i]);
                }

                if (
                  session['margin_before_end'] &&
                  session['margin_after_start'].valueOf() < timePoint.valueOf() &&
                  timePoint.valueOf() < session['margin_before_end'].valueOf()
                ) {
                  results[session.id]['break'].push(data[i]);
                }

                if (
                  session['margin_before_end'] &&
                  session['point_max_shift'] &&
                  session['margin_before_end'].valueOf() <= timePoint.valueOf() &&
                  timePoint.valueOf() <= session['point_max_shift'].valueOf()
                ) {
                  results[session.id]['checkout'].push(data[i]);
                }

                if (!timePointNext || timePointNext.valueOf() > session['point_max_shift'].valueOf()) {
                  if (!results[session.id]['checkin'].length && results[session.id]['break'].length) {
                    results[session.id]['checkin'].unshift(results[session.id]['break'].shift());
                  }
                  if (!results[session.id]['checkout'].length && results[session.id]['break'].length) {
                    results[session.id]['checkout'].push(results[session.id]['break'].pop());
                  }
                  session = null;
                }
              }
              else {
                results[session.id] = results[session.id] || { checkin: [], break: [], checkout: [] };

                if (timePoint.valueOf() <= session['margin_after_start'].valueOf()) {
                  results[session.id]['checkin'].push(data[i]);
                }
                else {
                  results[session.id]['checkout'].push(data[i]);
                  if (!session['margin_after_end']) {
                    session['margin_after_end'] = new Date(data[i].start + marginMinutesDefault)
                  }
                  if (timePointNext && session['margin_after_end'].valueOf() > timePointNext.valueOf()) {
                    session['margin_after_end'] = timePointNext.valueOf() + marginMinutesDefault
                  }
                }

                if (!timePointNext || (session['margin_after_end'] && session['margin_after_end'].valueOf() < timePointNext.valueOf())) {
                  session = null;
                }
              }

            }
          }

          return results;
        };

        const highlightCheckin = () => {
          let list_timeline = calculatorTimeline(items.get(), groups.get())
          let list_submit = [];
          for (const uid in list_timeline) {
            for (const str_date in list_timeline[uid]) {

              if (list_timeline[uid][str_date]['checkin'].length) {
                let checkin = list_timeline[uid][str_date]['checkin'].shift()
                items.update({ id: checkin.id, ...{className: 'check-in'}});
                if (list_timeline[uid][str_date]['checkin'].length) {
                  let list_update = []
                  for (let i = 0; i < list_timeline[uid][str_date]['checkin'].length; i++) {
                    let checkin = list_timeline[uid][str_date]['checkin'][i]
                    list_update.push({ id: checkin.id, ...{className: ''}})
                    list_submit.push({
                      id: checkin.id,
                      uid: checkin.group,
                      start: checkin.start,
                    })
                  }
                  items.update(list_update);
                }
              }

              if (list_timeline[uid][str_date]['break'].length) {
                let list_update = []
                for (let i = 0; i < list_timeline[uid][str_date]['break'].length; i++) {
                  let point_break = list_timeline[uid][str_date]['break'][i]
                  list_update.push({ id: point_break.id, ...{className: ''} })
                  list_submit.push({
                    id: point_break.id,
                    uid: point_break.group,
                    start: point_break.start,
                  })
                }
                if (list_update.length) {
                  items.update(list_update);
                }
              }

              if (list_timeline[uid][str_date]['checkout'].length) {
                let checkout = list_timeline[uid][str_date]['checkout'].pop()
                items.update({ id: checkout.id, ...{className: 'check-out'}});
                if (list_timeline[uid][str_date]['checkout'].length) {
                  let list_update = []
                  for (let i = 0; i < list_timeline[uid][str_date]['checkout'].length; i++) {
                    let checkout = list_timeline[uid][str_date]['checkout'][i]
                    list_update.push({ id: checkout.id, ...{className: ''}})
                    list_submit.push({
                      id: checkout.id,
                      uid: checkout.group,
                      start: checkout.start,
                    })
                  }
                  items.update(list_update);
                }
              }

            }
          }
          if (list_submit.length) {
            $('[name="data_work_time"]').val(JSON.stringify(list_submit))
          }
          else {
            $('[name="data_work_time"]').val('')
          }
        }

        var csrfToken = '';
        csrfTokenLoader()

        const data = drupalSettings.fingerprint_timeline.data
        const data_group = drupalSettings.fingerprint_timeline.data_group
        const data_sheet = drupalSettings.fingerprint_timeline.data_sheet
        const data_group_sheet = drupalSettings.fingerprint_timeline.data_group_sheet
        const filter_mode = drupalSettings.fingerprint_timeline.filter_mode
        let container = this;
        let items = new vis.DataSet(data.concat(data_sheet));
        let groups = new vis.DataSet(data_group);
        highlightCheckin()
        let options = {
          groupOrder: 'content',
          verticalScroll: false,
          horizontalScroll: false,
          showWeekScale: false,
          margin: { item: { vertical: 7} },
          orientation: 'both',
          editable: {
            add: true,         // add new items by double tapping
            updateTime: true,  // drag items horizontally
            updateGroup: true, // drag items from one group to another
            remove: true,       // delete an item by tapping the delete button top right
            overrideItems: false  // allow these options to override item.editable
          },
          limitSize: false,
          stack: false,
          type: 'box',
          tooltip: {
            template: function (originalItemData, parsedItemData) {
              let tooltip = null;
              if (originalItemData?.title) {
                tooltip = `<span>${originalItemData.title}</span>`
              }
              return tooltip;
            }
          },
          showCurrentTime: true,
          locale: drupalSettings.path.currentLanguage || 'en',
          maxHeight: '500px',
          groupHeightMode: 'fixed',
          zoomMin: 1000 * 60 * 60 * 6,
          zoomMax: 1000 * 60 * 60 * 24 * 33,
          start: new Date(filter_mode.start.date),
          end: new Date(filter_mode.end.date),
          snap: function (date, scale, step) {
            let hour = 60 * 1000;
            return Math.round(date / hour) * hour;
          },
          onAdd: async function (item, callback) {
            if (item.end) {
              callback(null); // cancel updating the item
              return;
            }
            item.start = item.start.valueOf();
            item.content = moment(new Date(item.start)).format('HH:mm')
            item.className = 'is-updating'
            callback(item); // cancel updating the item
            let item_post = {...item}
            item_post.start /= 1000
            actionAddFingerprint(item_post).then((result) => {
              items.remove(item.id); // cancel updating the item
              if (result) {
                item.id = 'fingerprint-' + result.id
                item.origin_group = item.group
                if (item.className) item.className = item.className.replace('is-updating', '')
                items.add(item)
                highlightCheckin()
              }
            })
          },

          onMove: function (item, callback) {
            item.start = item.start.valueOf();
            item.content = moment(new Date(item.start)).format('HH:mm')
            item.className = 'is-updating'
            callback(item) // send back adjusted item
            let item_post = {...item}
            item_post.start /= 1000;
            actionEditFingerprint(item.id, item_post).then((result) => {
              if (result) {
                item.origin_start = result.start * 1000;
                if (item.className) item.className = item.className.replace('is-updating', '')
                callback(item) // send back adjusted item
                highlightCheckin()
              }
            })
          },

          onMoving: function (item, callback) {
            if (item.className && item.className.includes('is-updating')) return

            if (item.group !== item.origin_group) {
              item.group = item.origin_group;
            }

            if (item.end) {
              callback(null); // send back the (possibly) changed item
              return;
            }
            item.content = moment(new Date(item.start)).format('HH:mm')
            callback(item);
          },

          onUpdate: function (item, callback) {
            callback(null); // cancel updating the item
          },

          onRemove: function (item, callback) {
            if (!item.className && item.className.includes('is-updating')) return
            actionDeleteFingerprint(item.id).then((result) => {
              if (result) {
                callback(item) // send back adjusted item
                highlightCheckin()
              }
            })
          }
        }
        let timeline = new vis.Timeline(container, items, groups, options);

        function closestShift (data, value) {
          if (!data.length) return null
          return data.reduce((prev, curr) => {
            return Math.abs(curr.start - value) < Math.abs(prev.start - value) ? curr : prev;
          });
        }

        $('[name="type_filter"]').on('change', function () {
          let type_date = $(this).val();
          let date_filter = $('[name="date_filter"]');
          const mapFormat = { week: 'Y-[W]WW', month: 'Y-MM' }
          const current_filter = moment().format(mapFormat[type_date])
          date_filter.attr('type', type_date)
          date_filter.val(current_filter)
        })

      })
    }
  };
}(Drupal, jQuery, once));
