(function (Drupal, $, once) {
  Drupal.behaviors.fingerprint_work_timeline = {
    attach: function (context, settings) {
      $(once('fingerprint_work_timeline', '#timeline', context)).each(function () {

        let maxRangeCurrent = {};
        let list_submit = [];

        const csrfTokenLoader = async () => {
          await axios.get(Drupal.url('session/token')).then((response) => {
            csrfToken = response.data;
          }).then(() => {

          });
        }

        const actionAddFingerprint = async (data) => {
          try {
            const response = await axios.post(Drupal.url('api/fingerprint'), data, {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': csrfToken
              }
            });
            return response.data;
          } catch (error) {
            return false
          }
        }

        const actionEditFingerprint = async (id, data) => {
          id = id.replace('fingerprint-', '')
          try {
            const response = await axios.patch(Drupal.url('api/fingerprint/' + id), data, {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': csrfToken
              }
            });
            return response.data;
          } catch (error) {
            return false
          }
        }

        const actionDeleteFingerprint = async (id) => {
          id = id.replace('fingerprint-', '')
          try {
            const response = await axios.delete(Drupal.url('api/fingerprint/' + id), {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': csrfToken
              }
            });
            return true;
          } catch (error) {
            return false
          }
        }

        const calculatorTimeline = (data, groups) => {
          data.sort((a, b) => a.start - b.start);
          let results = {};
          let dataTable = [];
          let dataWokTime = [];
          if (groups.length) {
            for (let i = 0; i < groups.length; i++) {
              let data_process = data.filter(item => item.group == groups[i].id && item.type !== 'background')
              let data_group = data.filter(item => item.group == groups[i].id && item.type == 'background')
              if (data_process.length) {
                results[groups[i].id] = processTimelineShift(data_process, data_group)
              }
            }
          }
          return results;
        }

        const processTimelineShift = (data, group) => {
          let results = {};
          let session = null;
          let marginMinutesDefault = 1800000

          for (let i = 0; i < data.length; i++) {
            let timePoint = new Date(data[i].start);
            let timePointNext = data?.[i + 1]?.start ? new Date(data[i + 1].start) : null;

            if (!session) {
              let dayStart = new Date(data[i].start);
              dayStart.setHours(0, 0, 0, 0);

              let shifts = group.filter(
                item => item.start >= dayStart.valueOf() && item.start < dayStart.valueOf() + 86400000
              );

              let shift = closestShift(shifts, data[i].start);
              const marginMinutesStart = shift ? shift['margin_start'] : marginMinutesDefault;

              let maxShift = shift ? new Date(shift.start + (86400000 - marginMinutesStart)) : null;
              let timePointEnd = shift ? new Date(shift['end']) : null;
              let timePointEndMax = maxShift;

              let marginAfterStart = shift
                ? new Date(shift.start + marginMinutesStart)
                : new Date(timePoint.valueOf() + marginMinutesStart);

              let marginBeforeEnd = timePointEnd ? new Date(timePointEnd.valueOf() - marginMinutesStart) : null;
              let marginAfterEnd = timePointEnd ? new Date(timePointEnd.valueOf() + marginMinutesStart) : null;

              session = {
                id: crypto.randomUUID(),
                start: timePoint,
                end: timePointEnd,
                end_max: timePointEndMax,
                margin_after_start: marginAfterStart,
                margin_before_end: marginBeforeEnd,
                margin_after_end: marginAfterEnd,
                point_max_shift: timePointEndMax,
              };
            }

            if (session) {

              if (session['end']) {
                results[session.id] = results[session.id] || { checkin: [], break: [], checkout: [] };

                if (timePoint.valueOf() <= session['margin_after_start'].valueOf()) {
                  results[session.id]['checkin'].push(data[i]);
                }

                if (
                  session['margin_before_end'] &&
                  session['margin_after_start'].valueOf() < timePoint.valueOf() &&
                  timePoint.valueOf() < session['margin_before_end'].valueOf()
                ) {
                  results[session.id]['break'].push(data[i]);
                }

                if (
                  session['margin_before_end'] &&
                  session['point_max_shift'] &&
                  session['margin_before_end'].valueOf() <= timePoint.valueOf() &&
                  timePoint.valueOf() <= session['point_max_shift'].valueOf()
                ) {
                  results[session.id]['checkout'].push(data[i]);
                }

                if (!timePointNext || timePointNext.valueOf() > session['point_max_shift'].valueOf()) {
                  if (!results[session.id]['checkin'].length && results[session.id]['break'].length) {
                    results[session.id]['checkin'].unshift(results[session.id]['break'].shift());
                  }
                  if (!results[session.id]['checkout'].length && results[session.id]['break'].length) {
                    results[session.id]['checkout'].push(results[session.id]['break'].pop());
                  }
                  session = null;
                }
              }
              else {
                results[session.id] = results[session.id] || { checkin: [], break: [], checkout: [] };

                if (timePoint.valueOf() <= session['margin_after_start'].valueOf()) {
                  results[session.id]['checkin'].push(data[i]);
                }
                else {
                  results[session.id]['checkout'].push(data[i]);
                  if (!session['margin_after_end']) {
                    session['margin_after_end'] = new Date(data[i].start + marginMinutesDefault)
                  }
                  if (timePointNext && session['margin_after_end'].valueOf() > timePointNext.valueOf()) {
                    session['margin_after_end'] = timePointNext.valueOf() + marginMinutesDefault
                  }
                }

                if (!timePointNext || (session['margin_after_end'] && session['margin_after_end'].valueOf() < timePointNext.valueOf())) {
                  session = null;
                }
              }

            }
          }

          return results;
        };

        const dataHandle = () => {
          let data_result = []
          const all_items = items.get()
          let list_timeline = calculatorTimeline(all_items, groups.get())
          for (const uid in list_timeline) {
            for (const str_date in list_timeline[uid]) {
              let item_data = {};
              if (list_timeline[uid][str_date]['checkin'].length && list_timeline[uid][str_date]['checkout'].length) {
                const checkin = list_timeline[uid][str_date]['checkin'].shift()
                const checkout = list_timeline[uid][str_date]['checkout'].pop();
                const duration = moment.duration(checkout.start.valueOf() - checkin.start.valueOf(), 'milliseconds')

                item_data = {
                  'id': crypto.randomUUID(),
                  'content': Drupal.t(`@hoursh@minutesm`, {'@hours': Math.floor(duration.asHours()), '@minutes': duration.minutes()}),
                  'start': checkin.start.valueOf(),
                  'end': checkout.start.valueOf(),
                  'group': checkin['group'],
                  'origin_group': checkin['group'],
                  'type': 'range'
                }

                const shifts = all_items.filter(record => record.type === 'background' && record.group == item_data.group)
                const shift = closestShift(shifts, item_data.valueOf())

                const time = calculatorWork(item_data, shift);
                addStyleItem(item_data, time)
                updateDataSubmit(item_data, time)

                data_result.push(item_data)
              }
            }
          }
          // Update value submit.
          $('[name="data_work_time"]').val(JSON.stringify(list_submit))

          return data_result;
        }

        var csrfToken = '';
        csrfTokenLoader()

        const data = drupalSettings.fingerprint_work_timeline.data
        const data_group = drupalSettings.fingerprint_work_timeline.data_group
        const data_sheet = drupalSettings.fingerprint_work_timeline.data_sheet
        const data_group_sheet = drupalSettings.fingerprint_work_timeline.data_group_sheet
        const filter_mode = drupalSettings.fingerprint_work_timeline.filter_mode
        let container = this;
        let items = new vis.DataSet(data.concat(data_sheet));
        let groups = new vis.DataSet(data_group);
        let data_handle = dataHandle()
        items = new vis.DataSet(data_handle.concat(data_sheet));
        let options = {
          groupOrder: 'content',
          verticalScroll: false,
          horizontalScroll: false,
          showWeekScale: false,
          margin: { item: { vertical: 7} },
          orientation: 'both',
          editable: {
            add: false,         // add new items by double tapping
            updateTime: true,  // drag items horizontally
            updateGroup: true, // drag items from one group to another
            remove: false,       // delete an item by tapping the delete button top right
            overrideItems: false  // allow these options to override item.editable
          },
          limitSize: false,
          stack: false,
          type: 'box',
          tooltip: {
            template: function (originalItemData, parsedItemData) {
              let tooltip = null;
              if (originalItemData?.title) {
                tooltip = `<span>${originalItemData.title}</span>`
              }
              return tooltip;
            }
          },
          showCurrentTime: true,
          locale: drupalSettings.path.currentLanguage || 'en',
          maxHeight: '500px',
          groupHeightMode: 'fixed',
          zoomMin: 1000 * 60 * 60 * 6,
          zoomMax: 1000 * 60 * 60 * 24 * 33,
          start: new Date(filter_mode.start.date),
          end: new Date(filter_mode.end.date),
          snap: function (date, scale, step) {
            let hour = 60 * 1000;
            return Math.round(date / hour) * hour;
          },

          onMove: function (item, callback) {
            const all_items = items.get()

            const shifts = all_items.filter(record => record.type === 'background' && record.group == item.group)
            const shift = closestShift(shifts, item.start.valueOf())

            const time = calculatorWork(item, shift);

            updateDataSubmit(item, time)

            // Update value submit.
            $('[name="data_work_time"]').val(JSON.stringify(list_submit))

            callback(item);
          },

          onMoving: function (item, callback) {
            if (item.className && item.className.includes('is-updating')) return

            if (item.group !== item.origin_group) {
              item.group = item.origin_group;
            }

            if (item.end.valueOf() - item.start.valueOf() < 3600000) return;

            const duration = moment.duration(item.end.valueOf() - item.start.valueOf(), 'milliseconds')

            item.content = Drupal.t(`@hoursh@minutesm`, {'@hours': Math.floor(duration.asHours()), '@minutes': duration.minutes()})

            if (maxRangeCurrent.left && item.start.valueOf() <= maxRangeCurrent.left) {
              item.start = maxRangeCurrent.left;
              return;
            }
            if (maxRangeCurrent.right && item.end.valueOf() >= maxRangeCurrent.right) {
              item.end = maxRangeCurrent.right;
              return;
            }

            let shifts = items.get();
            shifts = shifts.filter(record => record.type === 'background' && record.group == item.group)
            let shift = closestShift(shifts, item.start.valueOf())

            const time = calculatorWork(item, shift);

            addStyleItem(item, time)

            callback(item);
          },

          onUpdate: function (item, callback) {
            callback(null); // cancel updating the item
          },
        }
        let timeline = new vis.Timeline(container, null, options);
        timeline.setGroups(groups);
        timeline.setItems(items);

        timeline.on('select', function (properties) {
          maxRangeCurrent = {}
          let item_select = items.get(properties.items[0])
          let list_item = items.get()
          list_item = list_item.filter(item => item.group == item_select.group && item.type !== 'background')
          let result = findNearestElements(list_item, item_select)
          if (result.smaller) {
            maxRangeCurrent.left = result.smaller.end
          }
          if (result.larger) {
            maxRangeCurrent.right = result.larger.start
          }
        });

        function closestShift (shifts, value) {
          if (!shifts.length) return null
          return shifts.reduce((prev, curr) => {
            return Math.abs(curr.start - value) < Math.abs(prev.start - value) ? curr : prev;
          });
        }

        function findNearestElements(array, target) {
          let smaller = null;
          let larger = null;

          array.forEach(item => {
            let item_end = item.end instanceof Date ? item.end.valueOf() : item.end
            let target_start = target.start instanceof Date ? target.start.valueOf() : target.start
            if (item_end < target_start) {
              if (!smaller || item_end > smaller.end) {
                let item_draft = $.extend(true, {}, item)
                item_draft.start = item_draft.start instanceof Date ? item_draft.start.valueOf() : item_draft.start
                item_draft.end = item_draft.end instanceof Date ? item_draft.end.valueOf() : item_draft.end
                smaller = item_draft;
              }
            }
            let item_start = item.start instanceof Date ? item.start.valueOf() : item.start
            let target_end = target.end instanceof Date ? target.end.valueOf() : target.end
            if (item_start > target_end) {
              if (!larger || item_start < larger.start) {
                let item_draft = $.extend(true, {}, item)
                item_draft.start = item_draft.start instanceof Date ? item_draft.start.valueOf() : item_draft.start
                item_draft.end = item_draft.end instanceof Date ? item_draft.end.valueOf() : item_draft.end
                larger = item_draft;
              }
            }
          });

          return { smaller, larger };
        }

        function calculatorWork (data, shift = null) {

          let total_break = shift ? shift.break_end - shift.break_start : null
          let total_shift = shift ? shift.end - shift.start : null
          let real_total_shift = total_break && total_shift ? total_shift - total_break : null;

          let total, normal, over = null
          total = data.end.valueOf() - data.start.valueOf();
          if (total_shift && total > total_shift) {
            normal = total_shift
            over = total - total_shift
          }
          else {
            normal = total
          }
          return {time: data.start.valueOf(), total: total, normal: normal, over: over}
        }

        function addStyleItem(item, time) {
          if (time.over) {
            let ratio_over = time.over / time.total * 100
            item.style = `background: linear-gradient(270deg, #FFBFBF ${ratio_over}%, #d5ddf6 0%);`

            const duration_over = moment.duration(time.over, 'milliseconds')
            item.content += ` (${Drupal.t(`@hoursh@minutesm`, {'@hours': Math.floor(duration_over.asHours()), '@minutes': duration_over.minutes()})})`
          }
          else {
            item.style = 'background: #d5ddf6'
          }
        }

        function updateDataSubmit(item, time) {
          let index_item_submit = list_submit.findIndex(record => record.id === item.id)
          let index_item_submit_over = list_submit.findIndex(record => record.id === 'over-' + item.id)

          let item_submit = index_item_submit >= 0 ? list_submit[index_item_submit] : null
          let item_submit_over = index_item_submit_over >= 0 ? list_submit[index_item_submit_over] : null

          if (item_submit) {
            item_submit.start = time.time
            item_submit.end = time.time + time.normal
          }
          else {
            list_submit.push({
              id: item.id,
              group: item.group,
              start: time.time,
              end: time.time + time.normal
            })
          }

          if (item_submit_over) {
            if (time.over) {
              item_submit_over.start = time.time + time.normal
              item_submit_over.end = time.time + time.normal + time.over
            }
            else {
              list_submit.splice(index_item_submit_over, 1)
            }
          }
          else {
            if (time.over) {
              const duration_over = moment.duration(time.over, 'milliseconds')
              if (index_item_submit_over >= 0) {
                list_submit.splice(index_item_submit_over + 1, 0, {
                  id: 'over-' + item.id,
                  group: item.group,
                  start: time.time + time.normal,
                  end: time.time + time.normal + time.over
                })
              }
              else {
                list_submit.push({
                  id: 'over-' + item.id,
                  group: item.group,
                  start: time.time + time.normal,
                  end: time.time + time.normal + time.over
                })
              }
            }
          }
        }

        $('[name="type_filter"]').on('change', function () {
          let type_date = $(this).val();
          let date_filter = $('[name="date_filter"]');
          const mapFormat = { week: 'Y-[W]WW', month: 'Y-MM' }
          const current_filter = moment().format(mapFormat[type_date])
          date_filter.attr('type', type_date)
          date_filter.val(current_filter)
        })

      })
    }
  };
}(Drupal, jQuery, once));
