(function (Drupal, $, once) {
  'use strict';

  Drupal.behaviors.fingerprint_timeline = {
    attach: function (context, settings) {
      $(once('fingerprint_timeline', '#work-time-timeline', context)).each(function () {
        let originalGroupId = {};
        // Function
        // return format date, example: "%Y-%m-%d"
        const getFormatDate = (date, format = "%Y-%m-%d") => {
          let year = date.getFullYear();
          let month = (date.getMonth() + 1).toString().padStart(2, '0');
          let day = date.getDate().toString().padStart(2, '0');
          let hours = date.getHours().toString().padStart(2, '0');
          let minutes = date.getMinutes().toString().padStart(2, '0');
          let seconds = date.getSeconds().toString().padStart(2, '0');
          let results = format;
          results = results.replaceAll('%Y', year).replaceAll('%m', month).replaceAll('%d', day);
          results = results.replaceAll('%H', hours).replaceAll('%i', minutes).replaceAll('%s', seconds);
          return results;
        }

        // create visualization
        let options = {
          groupOrder: 'content',
          verticalScroll: false,
          horizontalScroll: false,
          showWeekScale: false,
          margin: { item: { vertical: 7} },
          orientation: 'both',
          editable: {
            add: true,         // add new items by double tapping
            updateTime: true,  // drag items horizontally
            updateGroup: true, // drag items from one group to another
            remove: true,       // delete an item by tapping the delete button top right
            overrideItems: false  // allow these options to override item.editable
          },
          limitSize: false,
          stack: false,
          type: 'box',
          tooltip: {
            template: function (originalItemData, parsedItemData) {
              let tooltip = null;
              if (originalItemData?.title) {
                tooltip = `<span>${originalItemData.title}</span>`
              }
              return tooltip;
            }
          },
          showCurrentTime: true,
          locale: drupalSettings.path.currentLanguage || 'en',
          maxHeight: '500px',
          groupHeightMode: 'fixed',
          zoomMin: 1000 * 60 * 60 * 6,
          zoomMax: 1000 * 60 * 60 * 24 * 30 * 3,
          snap: function (date, scale, step) {
            let hour = 60 * 1000;
            return Math.round(date / hour) * hour;
          },
          onAdd: async function (item, callback) {
            if (item.end) {
              callback(null); // cancel updating the item
              return;
            }
            item.start = item.start.valueOf();
            item.content = item.content = getFormatDate(new Date(item.start), '%H:%i')
            callback(item) // send back adjusted item
          },

          onMove: function (item, callback) {
            originalGroupId[item.id] = item.group;
            item.start = item.start.valueOf();
            item.content = item.content = getFormatDate(new Date(item.start), '%H:%i')
            callback(item) // send back adjusted item
          },

          onMoving: function (item, callback) {
            if (item.group !== originalGroupId[item.id]) {
              item.group = originalGroupId;
            }

            if (item.end) {
              callback(null); // send back the (possibly) changed item
              return;
            }
            item.content = item.content = getFormatDate(new Date(item.start), '%H:%i')
            callback(item);
          },

          onUpdate: function (item, callback) {
            callback(null); // cancel updating the item
          },

          onRemove: function (item, callback) {
            callback(item) // send back adjusted item
          }
        };
        // Create table.
        let data = drupalSettings.data_timeline.data;
        let group = drupalSettings.data_timeline.group;

        // Create an empty DataSet.
        // This DataSet is used for two-way data binding with the Timeline.
        let container = document.getElementById('work-time-timeline');
        let items = new vis.DataSet(data);
        let groups = new vis.DataSet(group);
        let timeline = new vis.Timeline(container, items, groups, options);

        $('[type="submit"]').on('click', function (e) {
          const data = items.get()
          $('#fingerprint-data').val(JSON.stringify(data))
        })

      })
    }
  };
}(Drupal, jQuery, once));
