(function ($, Drupal, drupalSettings, once) {
  Drupal.behaviors.work_sheet_schedule = {
    attach: function (context, settings) {
      $(once('work_sheet_schedule', '.work-sheet-schedule', context)).each(function () {
        const view_id = $(this).data('view-id')
        const display_id = $(this).data('display-id')

        const data_group = drupalSettings[`${view_id}-${display_id}`].data
        const data = drupalSettings[`${view_id}-${display_id}`].data_work_schedule
        const shifts = drupalSettings[`${view_id}-${display_id}`].data_shifts
        const mode_filter = drupalSettings[`${view_id}-${display_id}`].mode_filter

        const addScheduleShift = async (data) => {
          try {
            const response = await axios.post(Drupal.url('ajax/schedule-work-sheet/add'), data, {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': ''
              }
            });
            return response.data;
          } catch (error) {
            return false
          }
        }

        const updateScheduleShift = async (data) => {
          try {
            const response = await axios.post(Drupal.url('ajax/schedule-work-sheet/update'), data, {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': ''
              }
            });
            return response.data;
          } catch (error) {
            return false
          }
        }

        const removeScheduleShift = async (data) => {
          try {
            const response = await axios.post(Drupal.url('ajax/schedule-work-sheet/remove'), data, {
              params: {
                _format: 'json'
              },
              headers: {
                'Content-Type': 'application/json',
                'X-CSRF-Token': ''
              }
            });
            return response.data;
          } catch (error) {
            return false
          }
        }

        let items = new vis.DataSet(data);
        let groups = new vis.DataSet(data_group);

        let options = {
          groupOrder: 'content',
          verticalScroll: false,
          horizontalScroll: false,
          showWeekScale: false,
          editable: {
            add: true,         // add new items by double tapping
            updateTime: true,  // drag items horizontally
            updateGroup: true, // drag items from one group to another
            remove: true,       // delete an item by tapping the delete button top right
            overrideItems: false  // allow these options to override item.editable
          },
          limitSize: false,
          stack: false,
          type: 'box',
          tooltip: {
            template: function (originalItemData, parsedItemData) {
              let tooltip = null;
              if (originalItemData?.title) {
                tooltip = `<span>${originalItemData.title}</span>`
              }
              return tooltip;
            }
          },
          showCurrentTime: true,
          locale: drupalSettings.path.currentLanguage || 'en',
          maxHeight: '500px',
          groupHeightMode: 'fixed',
          orientation: 'both',
          zoomMin: 1000 * 60 * 60 * 6,
          zoomMax: 1000 * 60 * 60 * 24 * 33,
          start: new Date(mode_filter.start.date),
          end: new Date(mode_filter.end.date),
          snap: function (date, scale, step) {
            let hour = 60 * 1000;
            return Math.round(date / hour) * hour;
          },
          onAdd: async function (item, callback) {
            callback(item)
            if (shifts.length) {
              let date = new Date(item.start)
              let list_shifts = []
              for (let i = 0; i < shifts.length; i++) {
                if (!shifts[i].start || !shifts[i].end) continue
                let shift = {...shifts[i]}
                const [hours_shift_in, minutes_shift_in] = shift.start.split(':')
                const [hours_shift_out, minutes_shift_out] = shift.end.split(':')
                const date_shift_start = date.setHours(hours_shift_in, minutes_shift_in)
                let date_shift_end = date.setHours(hours_shift_out, minutes_shift_out)
                if (date_shift_start.valueOf() > date_shift_end.valueOf()) {
                  date_shift_end = moment(date_shift_end).add(1, 'days')
                }
                shift.time_start = date_shift_start.valueOf()
                shift.time_end = date_shift_end.valueOf()
                list_shifts.push(shift)
              }

              let shift = closestShift(list_shifts, item.start)
              if (shift) {
                let list_post = [];
                const [hours_start, minutes_start] = shift.start.split(':')
                const [hours_end, minutes_end] = shift.end.split(':')
                let filter_start = moment(mode_filter.start.date.slice(0, -3))
                let filter_end = moment(mode_filter.end.date.slice(0, -3))
                if (filter_start.valueOf() < filter_end.valueOf()) {
                  while (filter_start.valueOf() < filter_end.valueOf()) {
                    let start = filter_start.clone().hour(hours_start).minute(minutes_start)
                    let end = filter_start.clone().hour(hours_end).minute(minutes_end)
                    if (start.valueOf() > end.valueOf()) {
                      end = end.add(1, 'days')
                    }
                    list_post.push({
                      group: item.group,
                      start: start.valueOf(),
                      end: end.valueOf(),
                      margin_start: shift['margin_start'],
                      margin_end: shift['margin_end'],
                      time_start: shift.start,
                      time_end: shift.end,
                    })
                    filter_start = filter_start.add(1, 'days')
                  }
                }
                addScheduleShift(list_post).then((result) => {
                  items.add(Object.values(result))
                }).finally(() => {
                  items.remove(item.id)
                })
              }
            }

          },

          onMove: function (item, callback) {
            item.start = item.start.valueOf();
            item.content = moment(new Date(item.start)).format('HH:mm')
            item.className = 'is-updating'
            let date = new Date(item.start)
            let list_shifts = []
            if (shifts.length) {
              for (let i = 0; i < shifts.length; i++) {
                if (!shifts[i].start || !shifts[i].end) continue
                let shift = {...shifts[i]}

                // Time shift.
                const [hours_shift_in, minutes_shift_in] = shift.start.split(':')
                const [hours_shift_out, minutes_shift_out] = shift.end.split(':')
                const date_shift_start = new Date(date.valueOf()).setHours(hours_shift_in, minutes_shift_in)
                let date_shift_end = new Date(date.valueOf()).setHours(hours_shift_out, minutes_shift_out)
                if (date_shift_start.valueOf() > date_shift_end.valueOf()) {
                  date_shift_end = moment(date_shift_end).add(1, 'days')
                }

                shift.time_start = date_shift_start.valueOf()
                shift.time_end = date_shift_end.valueOf()
                list_shifts.push(shift)
              }
            }
            let shift = closestShift(list_shifts, new Date(item.start))
            if (shift) {
              item.className = 'is-updating'
              item.start = shift.time_start
              item.end = shift.time_end
              const hours_start_shift = moment(new Date(shift.time_start)).format('HH:mm')
              const hours_end_shift = moment(new Date(shift.time_end)).format('HH:mm')
              const hours_start_shift_break = shift.break_start
              const hours_end_shift_break = shift.break_end
              item.content = moment(new Date(item.start)).format('HH:mm')
              let list_item = items.get()
              list_item = list_item.filter(record => record.group == item.group)
              list_item.forEach(record => {

                // Time shift.
                const [hours_start, minutes_start] = hours_start_shift.split(':')
                const [hours_end, minutes_end] = hours_end_shift.split(':')
                const record_start = new Date(record.start).setHours(hours_start, minutes_start)
                let record_end = new Date(record.start).setHours(hours_end, minutes_end)
                if (record_start.valueOf() > record_end.valueOf()) {
                  record_end = moment(record_end).add(1, 'days')
                }

                // Time break
                const [hours_break_start, minutes_break_start] = hours_start_shift_break ? hours_start_shift_break.split(':') : [null]
                const [hours_break_end, minutes_break_end] = hours_end_shift_break ? hours_end_shift_break.split(':') : [null]
                let record_break_start = hours_start_shift_break ? new Date(record.start).setHours(hours_break_start, minutes_break_start) : null
                let record_break_end = hours_end_shift_break ? new Date(record.start).setHours(hours_break_end, minutes_break_end) : null
                if (record_break_start && record_start.valueOf() > record_break_start.valueOf()) {
                  record_break_start = moment(record_break_start).add(1, 'days')
                }
                if (record_break_end && record_start.valueOf() > record_break_end.valueOf()) {
                  record_break_end = moment(record_break_end).add(1, 'days')
                }
                if (record_break_start && record_break_end && record_break_start.valueOf() > record_break_end.valueOf()) {
                  let draft_break = new Date(record_break_start.valueOf())
                  record_break_start = new Date(record_break_end.valueOf())
                  record_break_end = draft_break
                }

                record.className = 'is-updating'
                record.start = record_start.valueOf()
                record.end = record_end.valueOf()
                record.break_start = record_break_start ? record_break_start.valueOf() : null
                record.break_end = record_break_end ? record_break_end.valueOf() : null
                record.content = moment(new Date(record.start)).format('HH:mm')
                record.margin_start = shift['margin_start']
                record.margin_end = shift['margin_end']
              })
              items.update(list_item);
              if (list_item.length) {
                updateScheduleShift(list_item).then((result) => {
                  list_item.forEach(record => {
                    if (result[record.id]) {
                      if (record.className) record.className = record.className.replace('is-updating', '')
                    }
                  })
                  items.update(list_item);
                })
              }
              callback(item);
            }
            else {
              callback(null);
            }
          },

          onMoving: function (item, callback) {
            if ((item.id + 0) > 0) {
              if (item.group !== item.origin_group) {
                item.group = item.origin_group;
              }

              let date_origin = moment(new Date(item.origin_start)).format('YYYY-MM-DD')
              let date = moment(new Date(item.start)).format('YYYY-MM-DD')

              if (date !== date_origin) {
                callback(null);
              }
              else {
                item.content = moment(new Date(item.start)).format('HH:mm')
                callback(item);
              }
            }
            else {
              callback(item);
            }
          },

          onUpdate: function (item, callback) {
            callback(null); // cancel updating the item
          },

          onRemove: function (item, callback) {
            let list_item = items.get()
            list_item = list_item.filter(record => record.group == item.group)
            removeScheduleShift(list_item).then((result) => {
              items.remove(Object.keys(result))
              callback(item) // send back adjusted item
            })
          }
        }

        let timeline = new vis.Timeline(this, items, groups, options);

        function closestShift(shifts, value) {
          if (!shifts.length) return null
          return shifts.reduce((prev, curr) => {
            return Math.abs(curr.time_start - value) < Math.abs(prev.time_start - value) ? curr : prev;
          });
        }
      });

    },
  };
}(jQuery, Drupal, drupalSettings, once));
