<?php

namespace Drupal\work_sheet\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Defines work-sheet Controller class.
 */
class WorkSheetController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public function scheduleWorkSheet($action, Request $request) {
    $data = $request->getContent();
    $data = Json::decode($data);
    $workSheetManage = $this->entityTypeManager()->getStorage('work_sheet');
    $result = [];
    $index = 0;
    if ($action == 'add') {
      $userManage = $this->entityTypeManager->getStorage('user');
      foreach ($data as $item) {
        $user = $userManage->load($item['group']);
        $entity = $workSheetManage->create([
          'label' => $this->t('Work sheet - @name [@start-@end]', [
            '@name' => $user->getDisplayName(),
            '@start' => $item['time_start'],
            '@end' => $item['time_end'],
          ]),
          'uid' => $item['group'],
          'created' => $item['start'] / 1000,
          'stopped' => $item['end'] / 1000,
          'margin_start' => $item['margin_start'] * 60,
          'margin_end' => $item['margin_end'] * 60,
          'break_start' => !empty($item['break_start']) ? $item['break_start'] / 1000 : NULL,
          'break_end' => !empty($item['break_end']) ? $item['break_end'] / 1000 : NULL,
        ]);
        $entity->save();
        $result[$entity->id()] = [
          'id' => $entity->id(),
          'content' => date('H:s', $entity->get('created')->value),
          'title' => date('H:i', $entity->get('created')->value),
          'group' => $entity->getOwnerId(),
          'origin_group' => $entity->getOwnerId(),
          'start' => $entity->get('created')->value * 1000,
          'end' => $entity->get('stopped')->value * 1000,
          'origin_start' => $entity->get('created')->value * 1000,
          'origin_end' => $entity->get('stopped')->value * 1000,
          'type' => 'range',
        ];
      }
    }
    else {
      $entities = $workSheetManage->loadMultiple(array_column($data, 'id'));
      foreach ($entities as $entity) {
        switch ($action) {
          case 'remove':
            $entity->set('status', 0);
            break;

          default:
            $entity->set('created', $data[$index]['start'] / 1000);
            $entity->set('stopped', $data[$index]['end'] / 1000);
            $entity->set('margin_start', $data[$index]['margin_start'] * 60);
            $entity->set('margin_end', $data[$index]['margin_end'] * 60);
            $entity->set('break_end', !empty($data[$index]['break_end']) ? $data[$index]['break_end'] / 1000 : NULL);
            $entity->set('break_start', !empty($data[$index]['break_start']) ? $data[$index]['break_start'] / 1000 : NULL);
        }
        $entity->save();
        $result[$entity->id()] = $entity->id();
        $index++;
      }
    }
    return new JsonResponse($result);
  }

}
